@@ -1,11 +1,11 @@
-# $Id: SAX.pm,v 1.1 2002/05/20 10:38:42 phish Exp $
+# $Id: SAX.pm,v 1.3 2002/06/21 11:33:35 phish Exp $
 
 package XML::LibXML::SAX;
 
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '0.01';
+$VERSION = '1.00';
 
 use XML::LibXML;
 use XML::SAX::Base;
@@ -40,10 +40,9 @@ sub _parse_string {
 
 sub _parse_systemid {
     my $self = shift;
-    my $fh = IO::File->new(shift);
     $self->{ParserOptions}{LibParser}      = XML::LibXML->new;
-    $self->{ParserOptions}{ParseFunc}      = \&XML::LibXML::parse_fh;
-    $self->{ParserOptions}{ParseFuncParam} = $fh;
+    $self->{ParserOptions}{ParseFunc}      = \&XML::LibXML::parse_file;
+    $self->{ParserOptions}{ParseFuncParam} = shift;
     return $self->_parse;
 }
 
