@@ -1,4 +1,4 @@
-# $Id: Builder.pm,v 1.16 2002/06/11 15:33:19 phish Exp $
+# $Id: Builder.pm,v 1.22 2002/06/25 18:16:54 phish Exp $
 
 package XML::LibXML::SAX::Builder;
 
@@ -103,7 +103,13 @@ sub start_element {
     # build namespaces
     foreach my $p ( $self->{NamespaceStack}->get_declared_prefixes() ) {
         my $uri = $self->{NamespaceStack}->get_uri($p);
-        $node->setNamespace($uri, $p, $uri eq $el->{NamespaceURI} ? 1 : 0 );
+        my $nodeflag = 0;
+        if ( defined $uri
+             and defined $el->{NamespaceURI}
+             and $uri eq $el->{NamespaceURI} ) {
+            $nodeflag = 1;
+        }
+        $node->setNamespace($uri, $p, $nodeflag );
     }
 
     # append
@@ -122,9 +128,13 @@ sub start_element {
     foreach my $key (keys %{$el->{Attributes}}) {
         my $attr = $el->{Attributes}->{$key};
         if (ref($attr)) {
-            next if defined $attr->{Prefix}
-                    and $attr->{Prefix} eq "xmlns"
-                    and $self->{USENAMESPACESTACK} == 1;
+            # catch broken name/value pairs
+            next unless $attr->{Name} ;
+            next if $self->{USENAMESPACESTACK}
+                    and ( $attr->{Name} eq "xmlns"
+                          or ( defined $attr->{Prefix}
+                               and $attr->{Prefix} eq "xmlns" ) );
+
 
             if ( defined $attr->{Prefix}
                  and $attr->{Prefix} eq "xmlns" ) {
